package com.ceb2b2000.connector.connectors;

import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.ceb2b2000.connector.service.ContentConnectionLocalServiceUtil;
import com.ceb2b2000.connector.util.ConnectableCollectionUtil;
import com.ceb2b2000.connector.util.ConnectorWebKeys;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;

public abstract class AbstractConnector<T> implements Connector<T> {
	
	public void initOnAction(ActionRequest request, ActionResponse response) {
		// do nothing ...
	}

	public void initOnRender(RenderRequest request, RenderResponse response) {
		// do nothing ...
	}

	public String getId() {
		return String.format(
				_CONNECTOR_NAME_TEMPLATE, getEntityClass().getName());
	}

	public SearchResult getConnectedTo(
			ConnectorContext ctx, long classPK, int from, int to)
		throws ConnectorException {

		try {
			PortletRequest request = ctx.getRequest();
			
			ThemeDisplay themeDisplay =	(ThemeDisplay) request.getAttribute(
					WebKeys.THEME_DISPLAY);
			
			long classNameId = PortalUtil.getClassNameId(getEntityClass());
			
			Hits hits = ContentConnectionLocalServiceUtil.search(
					themeDisplay.getCompanyId(), themeDisplay.getScopeGroupId(),
					0L, 0L,
					classPK, classNameId,
					0L, 0L,
					getSearchText(ctx), from, to);
			
			List<Connectable> entities =
				ConnectableCollectionUtil.asSecondaryEntityList(
						ctx, hits.getDocs(), getEntityFactory());
			
			return new SearchResult(hits.getLength(), entities);
		} catch (SystemException e) {
			throw new ConnectorException(e);
		} catch (PortalException e) {
			throw new ConnectorException(e);
		}
	}

	protected String getSearchText(ConnectorContext ctx) {
		return ctx.getRequest().getParameter(ConnectorWebKeys.SEARCH_TEXT);
	}

	public Connectable findEntity(ConnectorContext ctx, long entityId)
			throws ConnectorException {

		try {
			return getEntityFactory().fromModelId(ctx, entityId, 0L);
		} catch (PortalException e) {
			throw new ConnectorException(e);
		} catch (SystemException e) {
			throw new ConnectorException(e);
		}
	}

	public SearchResult getConnected(ConnectorContext ctx, int from, int to)
			throws ConnectorException {
		try {
			String keywords = getSearchText(ctx);

			long companyId = PortalUtil.getCompanyId(ctx.getRequest());
			long groupId = PortalUtil.getScopeGroupId(ctx.getRequest());
			long userId = PortalUtil.getUserId(ctx.getRequest());
			long ownerUserId = 0L; // don't care
			long classNameId = PortalUtil.getClassNameId(getEntityClass());
			
			Hits hits = ContentConnectionLocalServiceUtil.search(companyId,
					groupId, userId, ownerUserId, classNameId, keywords,
					from, to);

			Document[] docs = hits.getDocs();
			
			List<Connectable> entities = 
				ConnectableCollectionUtil.fromConnectionHits(ctx, docs);
			
			return new SearchResult(hits.getLength(), entities);
		} catch (SystemException e) {
			throw new ConnectorException(e);
		} catch (PortalException e) {
			throw new ConnectorException(e);
		}
	}

	
	public String getCustomForm() {
		return null; // use default form.
	}

	private static final String _CONNECTOR_NAME_TEMPLATE = "%s.connector.name";

}
