package com.ceb2b2000.connector.connectors;

import com.ceb2b2000.connector.model.ContentConnection;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.util.PortalUtil;

public final class ConnectableUtil {
	
	private ConnectableUtil() {}
	
	public static final Connectable getPrimaryEntity(
			ConnectorContext ctx, ContentConnection connection) 
		throws ConnectorException, PortalException, SystemException {
		
		Class<?> primaryClass = getClassForId(connection.getPrimaryClassId());
		
		long primaryId = connection.getPrimaryId();
		
		Connectable entity = getEntity(
				ctx, primaryId, primaryClass, connection.getPriority());
		
		return entity;
	}
	
	public static final Connectable getSecondaryEntity(
			ConnectorContext ctx, ContentConnection connection) 
		throws ConnectorException, PortalException, SystemException {
		
		Class<?> secondaryClass = getClassForId(
				connection.getSecondaryClassId());

		long secondaryId = connection.getSecondaryId();

		Connectable entity = getEntity(
				ctx, secondaryId, secondaryClass, connection.getPriority());

		return entity;
	}

	private static Connectable getEntity(ConnectorContext ctx,
			long entityId, Class<?> entityClass, long priority) 
		throws PortalException,	SystemException {
		
		Connector<?> secondaryConnector = ConnectorUtil.forClass(entityClass);

		ConnectableFactory<?> factory = secondaryConnector.getEntityFactory();

		Connectable entity = factory.fromModelId(ctx, entityId, priority);
		
		return entity;
	}

	private static final Class<?> getClassForId(long classNameId)
			throws ConnectorException {
		
		String className = PortalUtil.getClassName(classNameId);

		try {
			return Class.forName(className);
		} catch (ClassNotFoundException e) {
			if (_log.isErrorEnabled()) {
				_log.error(String.format(
						"class with ID '%s' and name '%s' not found",
						classNameId, className), e);
			}
			throw new ConnectorException(e);
		}
	}
	
	private static final Log _log = LogFactoryUtil.getLog(
			ConnectableUtil.class);
}
