package com.ceb2b2000.connector.connectors;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.ceb2b2000.connector.util.PropertiesUtil;

public class ConnectorConfiguration {
	
	public ConnectorConfiguration() throws IOException {
		this(PropertiesUtil.load("/connector.properties"));
	}
	
	public ConnectorConfiguration(Properties properties) {
		_supportedTypes = Collections.unmodifiableCollection(
				PropertiesUtil.getSequence(properties, _CONNECTOR_SUPPORTED_TYPES));
		
		_connectors = new HashMap<String, String>(_supportedTypes.size());
		
		for (String contentType: _supportedTypes) {
			String propertyName = String.format(
					"%s.connector.class", contentType);
			
			String connectorClass = properties.getProperty(propertyName);
			
			_connectors.put(contentType, connectorClass);
		}
	}
	
	public Collection<String> getSupportedTypes() {
		return _supportedTypes;
	}
	
	public String getConnectorForClass(String className) {
		String connector = _connectors.get(className);
		
		return connector == null ? null : connector.trim();
	}

	
	private final Collection<String> _supportedTypes;
	private final Map<String, String> _connectors;

	private static final String _CONNECTOR_SUPPORTED_TYPES =
		"content.connector.supported.types";
}
