package com.ceb2b2000.connector.connectors.impl;

import java.util.Date;

import com.ceb2b2000.connector.connectors.AbstractConnectable;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.Connector;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portlet.blogs.model.BlogsEntry;
import com.liferay.portlet.blogs.service.BlogsEntryLocalServiceUtil;

public class BlogsEntryConnectableFactory
	implements ConnectableFactory<BlogsEntry> {

	public BlogsEntryConnectableFactory(Connector<BlogsEntry> connector) {
		_connector = connector;
	}
	
	private static final class BlogsEntryConnectable 
		extends AbstractConnectable {

		public BlogsEntryConnectable(
				Connector<BlogsEntry> connector, BlogsEntry model, long priority) {
			_connector = connector;
			_model = model;
			_priority = priority;
		}
		
		public String getConnectorId() {
			return _connector.getId();
		}

		public Date getCreateDate() {
			return _model.getCreateDate();
		}

		public String getDescription() {
			return StringUtil.shorten(_model.getContent(), 50);
		}

		public Class<?> getEntityClass() {
			return _connector.getEntityClass();
		}

		public long getId() {
			return _model.getEntryId();
		}

		public String getKeywords() {
			return _model.getContent();
		}

		public Date getModifiedDate() {
			return _model.getModifiedDate();
		}

		public String getName() {
			return _model.getTitle();
		}

		public long getPriority() {
			return _priority;
		}
		
		private final BlogsEntry _model;
		private final Connector<BlogsEntry> _connector;
		private final long _priority;
	}
	
	public Connectable fromHit(ConnectorContext ctx, Document doc, long priority)
			throws PortalException, SystemException {
		
		long modelId = Long.parseLong(doc.get(Field.ENTRY_CLASS_PK));
		
		return fromModelId(ctx, modelId, priority);
	}

	public Connectable fromModel(
			ConnectorContext ctx, BlogsEntry model,	long priority) {

		return new BlogsEntryConnectable(_connector, model, priority);
	}

	public Connectable fromModelId(
			ConnectorContext ctx, long modelId,	long priority) 
		throws PortalException, SystemException {
		
		BlogsEntry model = BlogsEntryLocalServiceUtil.getBlogsEntry(modelId);
		
		return fromModel(ctx, model, priority);
	}

	private final Connector<BlogsEntry> _connector;
}
