package com.ceb2b2000.connector.connectors.impl;

import java.util.List;

import javax.portlet.PortletRequest;

import com.ceb2b2000.connector.connectors.AbstractConnector;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.ceb2b2000.connector.connectors.SearchResult;
import com.ceb2b2000.connector.util.ConnectableCollectionUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portlet.blogs.model.BlogsEntry;
import com.liferay.portlet.blogs.service.BlogsEntryLocalServiceUtil;

public class BlogsEntryConnector extends AbstractConnector<BlogsEntry> {

	public SearchResult getAvailable(ConnectorContext ctx, int from, int to,
			Long... excluded) throws ConnectorException {
		
		try {
			PortletRequest request = ctx.getRequest();
			
			long companyId = PortalUtil.getCompanyId(request);
			long groupId = PortalUtil.getScopeGroupId(request);
			long userId = PortalUtil.getUserId(request);
			long ownerUserId = 0; // any user, we don't care.
			String keywords = getSearchText(ctx);
			int start = from;
			int end = to;
			
			Hits hits =	BlogsEntryLocalServiceUtil.search(
					companyId, groupId, userId, ownerUserId, keywords, 
					start, end);
			
			ConnectableFactory<BlogsEntry> factory = getEntityFactory();
			
			List<Connectable> entities =
				ConnectableCollectionUtil.asConnectables(
						ctx, hits.getDocs(), factory, excluded);
			
			SearchResult result = new SearchResult(hits.getLength(), entities);
			
			return result;
		} catch (SystemException e) {
			throw new ConnectorException(e);
		} catch (PortalException e) {
			throw new ConnectorException(e);
		}
	}

	public Class<BlogsEntry> getEntityClass() {
		return BlogsEntry.class;
	}

	public ConnectableFactory<BlogsEntry> getEntityFactory() {
		return new BlogsEntryConnectableFactory(this);
	}

}
