package com.ceb2b2000.connector.connectors.impl;

import java.util.Date;

import javax.portlet.PortletURL;

import com.ceb2b2000.connector.connectors.AbstractConnectable;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.Connector;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.language.LanguageUtil;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.util.PortalClassInvoker;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portlet.calendar.model.CalEvent;
import com.liferay.portlet.calendar.service.CalEventLocalServiceUtil;

public class CalEventConnectableFactory implements ConnectableFactory<CalEvent> {
	
	private static final class CalEventConnectable extends AbstractConnectable {
		public CalEventConnectable(
				CalEvent event, String languageId, long priority, 
				Connector<CalEvent> connector) {
			
			_calEvent = event;
			_languageId = languageId;
			_connector = connector;
			_priority = priority;
		}
		
		public long getId() {
			return _calEvent.getEventId();
		}

		
		public String getName() {
			try {
				String result = _calEvent.getTitle();

				return StringUtil.shorten(result, _MAX_LENGTH);
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		public String getDescription() {
			try {
				String result = _calEvent.getDescription();
				
				return StringUtil.shorten(result, _MAX_LENGTH);
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		public Date getCreateDate() {
			return _calEvent.getCreateDate();
		}

		public Date getModifiedDate() {
			return _calEvent.getModifiedDate();
		}

		public String getConnectorId() {
			return _connector.getId();
		}

		public String getKeywords() {
			StringBuilder buffer = new StringBuilder();
			
			buffer.append(_calEvent.getTitle())
				  .append(' ')
				  .append(_calEvent.getDescription());
			
			return buffer.toString();
		}

		public Class<?> getEntityClass() {
			return _connector.getEntityClass();
		}
		
		public long getPriority() {
			return _priority;
		}
		
		public PortletURL getURL(ConnectorContext ctx)
			throws ConnectorException {
			
			return PortalConnectorsUtil.getCalEventURL(_calEvent, ctx);
		}
		
		private final CalEvent _calEvent;
		private final String _languageId;
		private final Connector<CalEvent> _connector;
		private final long _priority;

		private static final String EVENT_CONTENT_UTIL_CLASS_NAME =
			"com.liferay.portlet.calendar.util.EventContentUtil";
	
		private static final int _MAX_LENGTH = 50;
	}

	public CalEventConnectableFactory(Connector<CalEvent> connector) {
		_connector = connector;
	}
	
	public Connectable fromHit(
			ConnectorContext ctx, Document doc, long priority) 
		throws PortalException, SystemException {
		
		final long eventId = Long.parseLong(doc.get(Field.ENTRY_CLASS_PK));
		
		return fromModelId(ctx, eventId, priority);
	}

	public Connectable fromModel(
			ConnectorContext ctx, CalEvent model, long priority) {
		return new CalEventConnectable(
				model, LanguageUtil.getLanguageId(ctx.getRequest()), priority,
				_connector);
	}

	public Connectable fromModelId(
			ConnectorContext ctx, long modelId, long priority) 
		throws PortalException, SystemException {
		
		CalEvent model = CalEventLocalServiceUtil.getCalEvent(modelId);
		
		return fromModel(ctx, model, priority);
	}
	
	private Connector<CalEvent> _connector;
}
