package com.ceb2b2000.connector.connectors.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.ceb2b2000.connector.connectors.ConnectorFactory;
import com.ceb2b2000.connector.connectors.ConnectorFactoryException;
import com.ceb2b2000.connector.connectors.Connector;
import com.ceb2b2000.connector.connectors.ConnectorConfiguration;

public class DefaultConnectorFactory implements ConnectorFactory {
	public DefaultConnectorFactory(ConnectorConfiguration connectorProperties) {
		_connectorProperties = connectorProperties;
		_byNameCache = new HashMap<String,Connector<?>>();
		_byClassCache = new HashMap<Class<?>,Connector<?>>();
		
		_fillCache();
	}
	
	public Connector<?> getConnector(String connectorId) {
		return _byNameCache.get(connectorId);
	}

	public Connector<?> getConnector(Class<?> contentClass) {
		return _byClassCache.get(contentClass);
	}

	public Collection<Connector<?>> getAll() {
		return _byNameCache.values();
	}
	
	@SuppressWarnings("unchecked")
	private Class<Connector<?>> _getClass(String connectorClassName) {
		try {
			return (Class<Connector<?>>) Class.forName(connectorClassName);
		} catch (ClassNotFoundException e) {
			throw new ConnectorFactoryException(e);
		}
	}
	
	private Connector<?> _createConnector(Class<Connector<?>> connectorClass) {
		try {
			return connectorClass == null ? null : connectorClass.newInstance();
		} catch (InstantiationException e) {
			throw new ConnectorFactoryException(e);
		} catch (IllegalAccessException e) {
			throw new ConnectorFactoryException(e);
		}
	}

	private final void _fillCache() {
		for (String supportedType: _connectorProperties.getSupportedTypes()) {
			Class<?> contentClass = _getClass(supportedType);
			Connector<?> connector = _getConnector(contentClass);
			if (connector != null) {
				_byNameCache.put(connector.getId(), connector);
				_byClassCache.put(contentClass, connector);
			}
		}
	}
	
	private final Connector<?> _getConnector(Class<?> contentClass) {
		String contentClassName = contentClass.getName();
		String connectorClassName = 
			_connectorProperties.getConnectorForClass(contentClassName);
		
		if (connectorClassName == null) {
			return null;
		} else {
			Connector<?> connector = 
				_createConnector(_getClass(connectorClassName));

			return connector;
		}
	}
	
	private final ConnectorConfiguration _connectorProperties;
	private final Map<String,Connector<?>> _byNameCache;
	private final Map<Class<?>,Connector<?>> _byClassCache;
}
