package com.ceb2b2000.connector.connectors.impl;

import java.util.Date;

import javax.portlet.PortletURL;

import com.ceb2b2000.connector.connectors.AbstractConnectable;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.Connector;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portlet.journal.model.JournalArticle;
import com.liferay.portlet.journal.service.JournalArticleLocalServiceUtil;

public class JournalArticleConnectableFactory implements ConnectableFactory<JournalArticle> {
	private static final class JournalArticleConnectableEntity
			extends AbstractConnectable {
		public JournalArticleConnectableEntity(
				Connector<JournalArticle> connector, JournalArticle model,
				long priority) {
			
			_connector = connector;
			_model = model;
			_priority = priority;
		}
		
		public String getConnectorId() {
			return _connector.getId();
		}

		public Date getCreateDate() {
			return _model.getCreateDate();
		}

		public String getDescription() {
			return _model.getDescription();
		}

		public Class<?> getEntityClass() {
			return _connector.getEntityClass();
		}

		public long getId() {
			return _model.getId();
		}

		public String getKeywords() {
			return _model.getTitle() + " " + _model.getDescription(); // TODO
		}

		public Date getModifiedDate() {
			return _model.getModifiedDate();
		}

		public String getName() {
			return _model.getTitle();
		}

		public long getPriority() {
			return _priority;
		}
		
		public PortletURL getURL(ConnectorContext ctx)
			throws ConnectorException {
			
			return PortalConnectorsUtil.getJournalArticleURL(_model, ctx);
		}

		private final Connector<JournalArticle> _connector;
		private final JournalArticle _model;
		private final long _priority;
	}
	
	public JournalArticleConnectableFactory(Connector<JournalArticle> connector) {
		_connector = connector;
	}
	
	public Connectable fromHit(
			ConnectorContext ctx, Document doc, long priority)
		throws PortalException, SystemException {
		
		long id = Long.parseLong(doc.get(Field.ENTRY_CLASS_PK));
		
		return fromModelId(ctx, id, priority);
	}

	public Connectable fromModel(
			ConnectorContext ctx, JournalArticle model, long priority) {
		
		return new JournalArticleConnectableEntity(_connector, model, priority);
	}

	public Connectable fromModelId(
			ConnectorContext ctx, long modelId, long priority)
		throws PortalException, SystemException {
		
		JournalArticle article = JournalArticleLocalServiceUtil.getArticle(
				modelId);
		
		return fromModel(ctx, article, priority);
	}

	private final Connector<JournalArticle> _connector;
}
