package com.ceb2b2000.connector.connectors.impl;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.ceb2b2000.connector.connectors.AbstractConnector;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.ceb2b2000.connector.connectors.SearchResult;
import com.ceb2b2000.connector.model.StructureRestriction;
import com.ceb2b2000.connector.service.StructureRestrictionLocalServiceUtil;
import com.ceb2b2000.connector.util.ConnectableCollectionUtil;
import com.ceb2b2000.connector.util.ConnectorWebKeys;
import com.ceb2b2000.connector.util.JournalArticleSearchUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.dao.search.DAOParamUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.PortalClassInvoker;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portal.util.PortletKeys;
import com.liferay.portlet.PortalPreferences;
import com.liferay.portlet.PortletPreferencesFactoryUtil;
import com.liferay.portlet.journal.model.JournalArticle;
import com.liferay.portlet.journal.service.JournalArticleLocalServiceUtil;
import com.liferay.support.tomcat.loader.PortalClassLoaderFactory;

public class JournalArticleConnector extends AbstractConnector<JournalArticle> {
	@Override
	public void initOnRender(RenderRequest request, RenderResponse response) {
		try {
			/*
			 * We need the value of the `JournalArticleImpl.TYPES' field to use
			 * it in the `web_content_search_form.jsp' view. 
			 */
			ClassLoader classLoader = PortalClassLoaderFactory.getClassLoader();
			
			Class<?> aClass = classLoader.loadClass(_ARTICLE_IMPL_CLASS);
			
			Field typesField = aClass.getDeclaredField(_TYPES_FIELD);
			
			String[] types = (String[]) typesField.get(null); // static field.
			
			request.setAttribute("articleTypes", Arrays.asList(types));
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		} catch (SecurityException e) {
			throw new RuntimeException(e);
		} catch (NoSuchFieldException e) {
			throw new RuntimeException(e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
		
		super.initOnRender(request, response);
	}

	
	@Override
	public String getCustomForm() {
		return "/html/web_content_search_form.jsp";
	}

	public SearchResult getAvailable(
			ConnectorContext ctx, int from, int to, Long... excluded)
		throws ConnectorException {

		return advancedSearch(ctx, from, to);
	}

	private SearchResult advancedSearch(ConnectorContext ctx, int from, int to)
			throws ConnectorException {
		try {
			PortletRequest request = ctx.getRequest();
			
			long groupId = PortalUtil.getScopeGroupId(request);
			long companyId = PortalUtil.getCompanyId(request);

			Connectable primary = (Connectable) request.getAttribute(
						ConnectorWebKeys.PRIMARY_ENTITY);
			
			String searchArticleId = DAOParamUtil.getString(
					request, "searchArticleId");
			String title = DAOParamUtil.getLike(request, "title");
			String description = DAOParamUtil.getLike(request, "description");
			String content = DAOParamUtil.getLike(request, "content");
			String type = DAOParamUtil.getString(request, "type");
			String[] templateIds = StringUtil.split(
					DAOParamUtil.getString(request, "templateId"));
			String status = DAOParamUtil.getString(request, "status");

			String orderByCol = ParamUtil.getString(request, "orderByCol");
			String orderByType = ParamUtil.getString(request, "orderByType");
			
			boolean andOperator = ParamUtil.getBoolean(request, "andOperator");
			
			PortalPreferences preferences = PortletPreferencesFactoryUtil
					.getPortalPreferences(request);

			String[] restrictions = getStructureRestrictions(
					ctx, companyId, groupId, primary);
			
			boolean approved = JournalArticleSearchUtil.isApproved(status, true);
			boolean expired = JournalArticleSearchUtil.isExpired(status, false);
			Date reviewDate = JournalArticleSearchUtil.getReviewDate(status);
			
			if (JournalArticleSearchUtil.orderByDefined(orderByCol, orderByType)) {
				preferences.setValue(
						PortletKeys.JOURNAL, "articles-order-by-col",
						orderByCol);
				preferences.setValue(
						PortletKeys.JOURNAL, "articles-order-by-type",
						orderByType);
			} 
			
			OrderByComparator orderByComparator = getOrderByComparator(
					preferences);
			
			double version = 1.0;
			Date displayDateGT = null;
			Date displayDateLT = null;
			
			int count =	JournalArticleLocalServiceUtil.searchCount(
					companyId, groupId, searchArticleId, version, title, 
					description, content, type, restrictions, templateIds, 
					displayDateGT, displayDateLT, approved, expired, 
					reviewDate, andOperator);
			
			List<JournalArticle> articles = JournalArticleLocalServiceUtil.search(
					companyId, groupId, searchArticleId, version, title, 
					description, content, type, restrictions, templateIds, 
					displayDateGT, displayDateLT, approved, expired, reviewDate, 
					andOperator, from, to, orderByComparator);
			
			List<Connectable> entities =
				ConnectableCollectionUtil.asConnectables(
						ctx, articles, getEntityFactory());
			
			return new SearchResult(count, entities);
		} catch (SystemException e) {
			throw new ConnectorException(e);
		} catch (PortalException e) {
			throw new ConnectorException(e);
		}
	}

	private OrderByComparator getOrderByComparator(PortalPreferences preferences) {
		try {
			// JournalUtil is in `portal-impl.jar'.
			return (OrderByComparator) PortalClassInvoker.invoke(
					"com.liferay.portlet.journal.util.JournalUtil",
					"getArticleOrderByComparator",
					preferences.getValue(
							PortletKeys.JOURNAL, "articles-order-by-col", "id"),
					preferences.getValue(
							PortletKeys.JOURNAL, "articles-order-by-type", "asc"));
		} catch (Exception e) {
			// `getArticleOrderByComparator()' doesn't throw any checked exceptions.
			throw new RuntimeException(e);
		}
	}

	private String[] getStructureRestrictions(
			ConnectorContext ctx, long companyId, long groupId,
			Connectable primary) 
		throws PortalException, SystemException {
		
		PortletRequest request = ctx.getRequest();
		
		boolean restricted = Boolean.parseBoolean(
				request.getPreferences().getValue("restricted", "false"));
		
		if (restricted && primary != null && 
				getEntityClass().equals(primary.getEntityClass())) {
			JournalArticle article = JournalArticleLocalServiceUtil.getArticle(
					primary.getId());
			
			Collection<StructureRestriction> restrictions = 
				StructureRestrictionLocalServiceUtil.getRestrictions(
						companyId, groupId, article.getStructureId());
			
			String[] rr = new String[restrictions.size()];
			Iterator<StructureRestriction> iterator = restrictions.iterator();
			
			for (int i = 0; i < rr.length; i++) { 
				rr[i] = iterator.next().getRelatedStructureId();
			}
			
			return rr;
		} else {
			// the primary isn't a `JournalArticle' instance.
			return _EMPTY_RESTRICTIONS;
		}
	}

	public Class<JournalArticle> getEntityClass() {
		return JournalArticle.class;
	}

	public ConnectableFactory<JournalArticle> getEntityFactory() {
		return new JournalArticleConnectableFactory(this);
	}
	
	private static final String _TYPES_FIELD = "TYPES";
	private static final String _ARTICLE_IMPL_CLASS =
		"com.liferay.portlet.journal.model.impl.JournalArticleImpl";
	private static final String[] _EMPTY_RESTRICTIONS = new String[0];

}
