package com.ceb2b2000.connector.connectors.impl;

import java.lang.reflect.Method;
import java.util.Date;

import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletURL;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;
import javax.servlet.http.HttpServletRequest;

import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.ceb2b2000.connector.util.NullPortletURL;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.DocumentSummary;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.util.PortalClassLoaderUtil;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.model.Layout;
import com.liferay.portal.model.User;
import com.liferay.portal.service.LayoutLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portlet.PortletURLFactoryUtil;
import com.liferay.portlet.calendar.model.CalEvent;
import com.liferay.portlet.expando.model.ExpandoBridge;
import com.liferay.portlet.journal.model.JournalArticle;

final class PortalConnectorsUtil {
	
	public static final PortletURL getUserURL(User model, ConnectorContext ctx)
		throws ConnectorException {

		return NullPortletURL.getInstance();
	}
	
	public static final PortletURL getJournalArticleURL(
			JournalArticle model, ConnectorContext ctx)
		throws ConnectorException {
		
		String[] tagsEntries = null;
		String[] tagsCategories = null;
		
		String indexerClass = "com.liferay.portlet.journal.util.Indexer";
			
		Document doc = _getDocument(
				indexerClass, "getArticleDocument",
				new Class[] {
						long.class, long.class, String.class,
						double.class, String.class, String.class,
						String.class, String.class, Date.class,
						String[].class, String[].class, ExpandoBridge.class
				},
				model.getCompanyId(), model.getGroupId(), model.getArticleId(), 
				model.getVersion(), model.getTitle(), model.getDescription(), 
				model.getContent(), model.getType(), model.getDisplayDate(), 
				tagsCategories, tagsEntries, model.getExpandoBridge());
		
		return _getURLFromDocument(ctx, indexerClass, doc);
	}

	public static final PortletURL getCalEventURL(
			CalEvent model, ConnectorContext ctx) throws ConnectorException {
		
		String[] tagsEntries = null;
			
		String indexerClass = "com.liferay.portlet.calendar.util.Indexer";
			
		Document doc = _getDocument(
				indexerClass, "getEventDocument",
				new Class[] { 
						long.class, long.class, long.class, String.class,
						long.class, String.class, String.class, Date.class,
						String[].class, ExpandoBridge.class
				},
				model.getCompanyId(), model.getGroupId(), 
				model.getUserId(), model.getUserName(), 
				model.getEventId(), model.getTitle(), 
				model.getDescription(), model.getCreateDate(), 
				tagsEntries, model.getExpandoBridge());
			
		return _getURLFromDocument(ctx, indexerClass, doc);
	}

	private static PortletURL _getURLFromDocument(
			ConnectorContext ctx, String indexerClassName, Document doc)
		throws ConnectorException {
		
		try {
			PortletURL url = _getPortletURL(
					ctx.getRequest(), doc.get(Field.PORTLET_ID),
					Long.parseLong(doc.get(Field.GROUP_ID)));
			
			DocumentSummary summary = _getSummary(indexerClassName, doc, url);
			
			return summary.getURL();
		} catch (ConnectorException e) {
			throw e;
		} catch (Exception e) {
			throw new ConnectorException(e);
		}
	}
	
	private static final DocumentSummary _getSummary(
			String className, Document doc, PortletURL url)
		throws ConnectorException {
		
		try {
			ClassLoader pcl = PortalClassLoaderUtil.getClassLoader();
			
			Class<?> aClass = pcl.loadClass(className);
			
			Object instance = aClass.newInstance();
			
			Method method = aClass.getMethod(
					"getDocumentSummary", Document.class, PortletURL.class);
			
			return (DocumentSummary) method.invoke(instance, doc, url);
		} catch (Exception e) {
			throw new ConnectorException(e);
		}
	}
	
	private static final Document _getDocument(
			String indexerClass, String methodName, Class<?>[] classes, 
			Object... args) throws ConnectorException {
		
		try {
			ClassLoader pcl = PortalClassLoaderUtil.getClassLoader();
			
			Class<?> aClass = pcl.loadClass(indexerClass);
			Method method = aClass.getMethod(methodName, classes);
			
			return (Document) method.invoke(null, args);
		} catch (Exception e) {
			throw new ConnectorException("couldn't invoke method on Indexer", e);
		}
	}
	
	private static final PortletURL _getPortletURL(
			PortletRequest portletRequest, String portletId, long groupId)
		throws PortletModeException, SystemException, WindowStateException {

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
				portletRequest);

		long plid = LayoutLocalServiceUtil.getDefaultPlid(
			groupId, false, portletId);

		if (plid == 0) {
			plid = LayoutLocalServiceUtil.getDefaultPlid(
				groupId, true, portletId);
		}

		if (plid == 0) {
			Layout layout = (Layout) request.getAttribute(WebKeys.LAYOUT);

			if (layout != null) {
				plid = layout.getPlid();
			}
		}

		
		PortletURL portletURL = PortletURLFactoryUtil.create(
			request, portletId, plid, PortletRequest.RENDER_PHASE);

		portletURL.setWindowState(WindowState.MAXIMIZED);
		portletURL.setPortletMode(PortletMode.VIEW);

		return portletURL;
	}
	
	private PortalConnectorsUtil() {}

}
