package com.ceb2b2000.connector.connectors.impl;

import java.util.Date;

import javax.portlet.PortletURL;

import com.ceb2b2000.connector.connectors.AbstractConnectable;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.Connector;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

public class UserConnectableFactory implements ConnectableFactory<User> {
	private static final class UserConnectable extends AbstractConnectable {
		public UserConnectable(
				Connector<User> connector, User model, long priority) {
			_connector = connector;
			_model = model;
			_priority = priority;
		}
		
		public String getConnectorId() {
			return _connector.getId();
		}

		public Date getCreateDate() {
			return _model.getCreateDate();
		}

		public String getDescription() {
			return _model.getFullName();
		}

		public Class<?> getEntityClass() {
			return _connector.getEntityClass();
		}

		public long getId() {
			return _model.getUserId();
		}

		public String getKeywords() {
			return _model.getFullName(); // TODO
		}

		public Date getModifiedDate() {
			return _model.getModifiedDate();
		}

		public String getName() {
			return _model.getFullName();
		}
		
		public long getPriority() {
			return _priority;
		}
		
		public PortletURL getURL(ConnectorContext ctx)
				throws ConnectorException {
			return PortalConnectorsUtil.getUserURL(_model, ctx);
		}


		private final Connector<User> _connector;
		private final User _model;
		private final long _priority;
	}
	
	public UserConnectableFactory(Connector<User> connector) {
		_connector = connector;
	}
	
	public Connectable fromHit(
			ConnectorContext ctx, Document doc, long priority)
		throws PortalException, SystemException {
		
		long userId = Long.parseLong(doc.get(Field.ENTRY_CLASS_PK));
		
		return fromModelId(ctx, userId, priority);
	}

	public Connectable fromModel(
			ConnectorContext ctx, User model, long priority) {
		
		return new UserConnectable(_connector, model, priority);
	}

	public Connectable fromModelId(
			ConnectorContext ctx, long modelId, long priority)
		throws PortalException, SystemException {
		
		User user = UserLocalServiceUtil.getUser(modelId);
		
		return fromModel(ctx, user, priority);
	}

	private final Connector<User> _connector;
}
