package com.ceb2b2000.connector.portlet;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;

import com.ceb2b2000.connector.NoSuchContentConnectionException;
import com.ceb2b2000.connector.NoSuchStructureRestrictionException;
import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.Connector;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.ceb2b2000.connector.connectors.ConnectorUtil;
import com.ceb2b2000.connector.connectors.impl.DefaultConnectorContext;
import com.ceb2b2000.connector.connectors.impl.EmptyConnector;
import com.ceb2b2000.connector.model.StructureRestriction;
import com.ceb2b2000.connector.service.ContentConnectionLocalServiceUtil;
import com.ceb2b2000.connector.service.StructureRestrictionLocalServiceUtil;
import com.ceb2b2000.connector.util.ConnectorWebKeys;
import com.ceb2b2000.connector.util.JournalStructureUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.KeyValuePair;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portlet.journal.model.JournalStructure;

public class ConnectorPortlet extends MultiViewJSPPortlet {
	@Override
	public void doView(RenderRequest request, RenderResponse response)
		throws IOException, PortletException {
		
		if (_log.isDebugEnabled()) {
			_log.debug("initializing connectors in view request");
		}

		Iterable<Connector<?>> connectors = ConnectorUtil.getAll();
		
		for (Connector<?> c: connectors) {
			c.initOnRender(request, response);
		}

		try {
			initializeCommonAttributes(request, response, connectors);
		} catch (Exception e) {
			throw new PortletException(e);
		}
		
		super.doView(request, response);
	}

	@Override
	public void processAction(ActionRequest request, ActionResponse response) 
		throws IOException, PortletException {
		
		if (_log.isDebugEnabled()) {
			_log.debug("initializing connectors in action request");
		}
		
		Iterable<Connector<?>> connectors = ConnectorUtil.getAll();
		
		for (Connector<?> c: connectors) {
			c.initOnAction(request, response);
		}

		try {
			initializeCommonAttributes(request, response, connectors);
		} catch (Exception e) {
			throw new PortletException(e);
		}
		
		super.processAction(request, response);
	}
	
	@Override
	public void doEdit(
			RenderRequest renderRequest, RenderResponse renderResponse) 
		throws IOException, PortletException {
		
		try {
			long companyId = PortalUtil.getCompanyId(renderRequest);
			long groupId = PortalUtil.getScopeGroupId(renderRequest);
			
			// list of all available structures. 
			
			Collection<JournalStructure> structures = 
				JournalStructureUtil.getAllStructures(companyId, groupId);
			
			renderRequest.setAttribute(
					ConnectorWebKeys.AVAILABLE_STRUCTURES, structures);
			
			// the selected structure in the structures select.
			
			String selectedStructureId = ParamUtil.getString(
					renderRequest, ConnectorWebKeys.SELECTED_STRUCTURE_ID);
			
			// if none selected, pick the first one.
			
			if (StringUtils.isBlank(selectedStructureId) &&
					!structures.isEmpty()) {
				JournalStructure str = structures.iterator().next();
				
				selectedStructureId = str.getStructureId();
			}
			
			renderRequest.setAttribute(
					ConnectorWebKeys.SELECTED_STRUCTURE_ID, selectedStructureId);
			
			// selected structure restrictions
			
			String[] currentIds = getRestrictionIds(
					companyId, groupId, selectedStructureId);
	
			fillSelectionLists(renderRequest, currentIds);
			
			// restrict web content relations?
			
			PortletPreferences prefs = renderRequest.getPreferences();
			
			boolean restrict = Boolean.parseBoolean(
					prefs.getValue("restrict-webcontent-relations", "false"));
			
			renderRequest.setAttribute(ConnectorWebKeys.RESTRICTED, restrict);
			
			super.doEdit(renderRequest, renderResponse);
		} catch (SystemException e) {
			throw new PortletException(e);
		}
	}

	private String[] getRestrictionIds(
			long companyId, long groupId, String structureId) 
		throws SystemException {
		
		List<StructureRestriction> restrictions =
			StructureRestrictionLocalServiceUtil.getRestrictions(
					companyId, groupId, structureId);
		
		String[] ids = new String[restrictions.size()];
		
		Iterator<StructureRestriction> iterator = restrictions.iterator();
		
		for (int i = 0; i < ids.length; i++) {
			ids[i] = iterator.next().getRelatedStructureId();
		}
		
		return ids;
	}

	private void fillSelectionLists(
			RenderRequest renderRequest, String[] currentIds) 
		throws SystemException {

		long companyId = PortalUtil.getCompanyId(renderRequest);
		long groupId = PortalUtil.getScopeGroupId(renderRequest);

		// get the list of the currently connected structures.
		
		List<JournalStructure> cc = JournalStructureUtil.getCurrentStructures(
				companyId, groupId, currentIds);

		List<KeyValuePair> current = new ArrayList<KeyValuePair>();
		
		for (JournalStructure structure: cc) {
			current.add(new KeyValuePair(
					structure.getStructureId(), structure.getName()));
		}

		// get the list of available structures.
		
		List<KeyValuePair> available = new ArrayList<KeyValuePair>();

		List<JournalStructure> availableStructures = 
			JournalStructureUtil.getAvailableStructures(
					companyId, groupId, currentIds);
		
		for (JournalStructure structure: availableStructures) {
			available.add(new KeyValuePair(
					structure.getStructureId(), structure.getName()));
		}
		
		renderRequest.setAttribute(ConnectorWebKeys.LEFT_STRUCTURES, current);
		renderRequest.setAttribute(ConnectorWebKeys.RIGHT_STRUCTURES, available);
	}

	protected void initializeCommonAttributes(
			PortletRequest request, PortletResponse response, 
			Iterable<Connector<?>> connectors) 
		throws Exception {
		
		// List of available connectors.
		
		request.setAttribute(ConnectorWebKeys.AVAILABLE_CONNECTORS, connectors);
		
		// Connector context.
		
		final DefaultConnectorContext ctx = new DefaultConnectorContext(
				request, response);
		
		request.setAttribute(ConnectorWebKeys.CONNECTOR_CONTEXT, ctx);
		
		// Selected connector (may be a primary or secondary).
		
		String selectedId = ParamUtil.getString(
				request, ConnectorWebKeys.SELECTED_CONNECTOR_ID);
		
		Connector<?> emptyConnector = EmptyConnector.getInstance();
		Connector<?> selectedConnector = ConnectorUtil.getWithId(selectedId);
		Connector<?> defaultConnector =
			connectors.iterator().hasNext() ? connectors.iterator().next() : emptyConnector;
		
		request.setAttribute(
				ConnectorWebKeys.SELECTED_CONNECTOR, 
				selectedConnector == null ? defaultConnector : selectedConnector);
	}

	public void editConnections(RenderRequest request, RenderResponse response) 
		throws PortletException	{
		
		// Primary connector.
		
		String connectorId = ParamUtil.getString(
				request, ConnectorWebKeys.PRIMARY_CONNECTOR_ID);
		
		Connector<?> primaryConnector = ConnectorUtil.getWithId(connectorId);
		
		Connector<?> emptyConnector = EmptyConnector.getInstance();
		
		request.setAttribute(
				ConnectorWebKeys.PRIMARY_CONNECTOR,
				primaryConnector == null ? emptyConnector : primaryConnector);
	
		// Primary entity.
		
		Long entityId = ParamUtil.getLong(
				request, ConnectorWebKeys.PRIMARY_ENTITY_ID);
		
		ConnectorContext ctx = (ConnectorContext) request.getAttribute(
				ConnectorWebKeys.CONNECTOR_CONTEXT);
		
		if (Validator.isNotNull(entityId)) {
			try {
				request.setAttribute(ConnectorWebKeys.PRIMARY_ENTITY,
									 primaryConnector.findEntity(ctx, entityId));
			} catch (ConnectorException e) {
				throw new PortletException(e);
			}
		}
		
		request.setAttribute(
				ConnectorWebKeys.SECONDARY_CONNECTOR,
				request.getAttribute(ConnectorWebKeys.SELECTED_CONNECTOR));
	}
	
	public void defaultView(RenderRequest request, RenderResponse response) {
		request.setAttribute(
				ConnectorWebKeys.PRIMARY_CONNECTOR,
				request.getAttribute(ConnectorWebKeys.SELECTED_CONNECTOR));
	}
	
	public void createConnection(ActionRequest request, ActionResponse response) 
		throws PortletException {

		try {
			// Connector context.
			ConnectorContext ctx = (ConnectorContext) request.getAttribute(
					ConnectorWebKeys.CONNECTOR_CONTEXT);
			
			// Primary connector.
				
			String connectorId = ParamUtil.getString(
					request, ConnectorWebKeys.PRIMARY_CONNECTOR_ID);
				
			Connector<?> primaryConnector = ConnectorUtil.getWithId(connectorId);
				
			// Primary entity.
				
			Long primaryEntityId = ParamUtil.getLong(
				request, ConnectorWebKeys.PRIMARY_ENTITY_ID);

			Class<?> primaryClass = primaryConnector.getEntityClass();
			
			Connectable primaryEntity = primaryConnector.findEntity(
					ctx, primaryEntityId);
			
			String primaryKeywords = primaryEntity.getKeywords();
			
			// Secondary connector.
				
			Connector<?> secondaryConnector =
				(Connector<?>) request.getAttribute(
						ConnectorWebKeys.SELECTED_CONNECTOR);
				
			// Secondary entity.
				
			Long secondaryEntityId = ParamUtil.getLong(
				request, ConnectorWebKeys.SECONDARY_ENTITY_ID);
			
			Class<?> secondaryClass = secondaryConnector.getEntityClass();
			
			Connectable secondaryEntity = secondaryConnector.findEntity(
					ctx, secondaryEntityId);
			
			String secondaryKeywords = secondaryEntity.getKeywords();
			
			// Audit data.
			
			ThemeDisplay themeDisplay =
				(ThemeDisplay) request.getAttribute(WebKeys.THEME_DISPLAY);
			
			long companyId = themeDisplay.getCompanyId();
			long groupId = themeDisplay.getScopeGroupId();
			long userId = themeDisplay.getUserId();
			
			ContentConnectionLocalServiceUtil.addContentConnection(
					companyId, groupId, userId,
					primaryEntityId, primaryClass, primaryKeywords,
					secondaryEntityId, secondaryClass, secondaryKeywords);
			
		} catch (SystemException e) {
			throw new PortletException(e);
		} catch (ConnectorException e) {
			throw new PortletException(e);
		}
	}
	
	public void deleteConnection(ActionRequest request, ActionResponse response)
		throws PortletException {
		
		try {
			// Primary connector.
			
			String primaryConnectorId = ParamUtil.getString(
					request, ConnectorWebKeys.PRIMARY_CONNECTOR_ID);
				
			Connector<?> primaryConnector = ConnectorUtil.getWithId(
					primaryConnectorId);
				
			// Primary entity.
				
			Long primaryEntityId = ParamUtil.getLong(
				request, ConnectorWebKeys.PRIMARY_ENTITY_ID);

			Class<?> primaryClass = primaryConnector.getEntityClass();
			
			// Secondary connector.
			
			String secondaryConnectorId = ParamUtil.getString(
					request, ConnectorWebKeys.SECONDARY_CONNECTOR_ID);
			Connector<?> secondaryConnector = ConnectorUtil.getWithId(
					secondaryConnectorId);
				
			// Secondary entity.
				
			Long secondaryEntityId = ParamUtil.getLong(
				request, ConnectorWebKeys.SECONDARY_ENTITY_ID);
			
			Class<?> secondaryClass = secondaryConnector.getEntityClass();
			
			ThemeDisplay themeDisplay =
				(ThemeDisplay) request.getAttribute(WebKeys.THEME_DISPLAY);
			
			long companyId = themeDisplay.getCompanyId();
			long groupId = themeDisplay.getScopeGroupId();
			
			ContentConnectionLocalServiceUtil.deleteContentConnection(
					companyId, groupId, primaryEntityId, primaryClass, 
					secondaryEntityId, secondaryClass);
		} catch (NoSuchContentConnectionException e) {
			throw new PortletException(e);
		} catch (SystemException e) {
			throw new PortletException(e);
		}
	}
	
	public void savePreferences(ActionRequest request, ActionResponse response) 
		throws NoSuchStructureRestrictionException, SystemException, 
			   PortletException, IOException {
		
		String currentStructs = ParamUtil.getString(
				request, ConnectorWebKeys.CURRENT_CONNECTIONS);
		
		String[] current = StringUtil.split(currentStructs);
		
		long companyId = PortalUtil.getCompanyId(request);
		long groupId = PortalUtil.getScopeGroupId(request);
		long userId = PortalUtil.getUserId(request);
		
		String structureId = ParamUtil.getString(
				request, ConnectorWebKeys.SELECTED_STRUCTURE_ID);
		
		StructureRestrictionLocalServiceUtil.updateRestrictions(
				companyId, groupId, userId, structureId, current);
		
		boolean restrict = ParamUtil.getBoolean(
				request, ConnectorWebKeys.RESTRICTED, false);
		
		PortletPreferences prefs = request.getPreferences();
		
		prefs.setValue(
				"restrict-webcontent-relations", Boolean.toString(restrict));
		prefs.store();
	}
	
	public void increasePriority(ActionRequest request, ActionResponse response)
		throws PortalException, SystemException {
		
		// primary entity
		
		long primaryId = ParamUtil.getLong(
				request, ConnectorWebKeys.PRIMARY_ENTITY_ID);
		String primaryConnectorId = ParamUtil.getString(
				request, ConnectorWebKeys.PRIMARY_CONNECTOR_ID);
		
		Connector<?> primaryConnector = ConnectorUtil.getWithId(
				primaryConnectorId);
		
		long primaryClassId = PortalUtil.getClassNameId(
				primaryConnector.getEntityClass());
		
		// secondary entity
		
		long secondaryId = ParamUtil.getLong(
				request, ConnectorWebKeys.SECONDARY_ENTITY_ID);
		String secondaryConnectorId = ParamUtil.getString(
				request, ConnectorWebKeys.SECONDARY_CONNECTOR_ID);
		
		Connector<?> secondaryConnector = ConnectorUtil.getWithId(
				secondaryConnectorId);

		long secondaryClassId = PortalUtil.getClassNameId(
				secondaryConnector.getEntityClass());
		
		long companyId = PortalUtil.getCompanyId(request);
		long groupId = PortalUtil.getScopeGroupId(request);
		
		ContentConnectionLocalServiceUtil.incrementPriority(
				companyId, groupId, primaryId, primaryClassId, secondaryId,
				secondaryClassId);
	}
	
	public void decreasePriority(ActionRequest request, ActionResponse response) 
		throws PortalException, SystemException {
		
		// primary entity
		
		long primaryId = ParamUtil.getLong(
				request, ConnectorWebKeys.PRIMARY_ENTITY_ID);
		String primaryConnectorId = ParamUtil.getString(
				request, ConnectorWebKeys.PRIMARY_CONNECTOR_ID);
		
		Connector<?> primaryConnector = ConnectorUtil.getWithId(
				primaryConnectorId);
		
		long primaryClassId = PortalUtil.getClassNameId(
				primaryConnector.getEntityClass());
		
		// secondary entity
		
		long secondaryId = ParamUtil.getLong(
				request, ConnectorWebKeys.SECONDARY_ENTITY_ID);
		String secondaryConnectorId = ParamUtil.getString(
				request, ConnectorWebKeys.SECONDARY_CONNECTOR_ID);
		
		Connector<?> secondaryConnector = ConnectorUtil.getWithId(
				secondaryConnectorId);

		long secondaryClassId = PortalUtil.getClassNameId(
				secondaryConnector.getEntityClass());
		
		long companyId = PortalUtil.getCompanyId(request);
		long groupId = PortalUtil.getScopeGroupId(request);
		
		ContentConnectionLocalServiceUtil.decrementPriority(
				companyId, groupId, primaryId, primaryClassId, secondaryId,
				secondaryClassId);
	}

	private static final Log _log =
		LogFactoryUtil.getLog(ConnectorPortlet.class);
}
