package com.ceb2b2000.connector.portlet;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;

import com.ceb2b2000.connector.util.StringUtil;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.MethodCache;
import com.liferay.portal.kernel.util.MethodKey;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.util.bridges.jsp.JSPPortlet;

public class MultiViewJSPPortlet extends JSPPortlet {
	public static final String REQUESTED_VIEW = "showView";
	public static final String DEFAULT_VIEW = "defaultView";
	
	@Override
	public void init() {
		super.init();
		
		_viewMappings = StringUtil.toMapping(viewJSP);
	}

	@Override
	public void doView(
			RenderRequest renderRequest, RenderResponse renderResponse)
		throws IOException, PortletException {
		
		String goTo = getViewName(renderRequest);
		
		callRenderMethod(renderRequest, renderResponse);
		
		include(goTo, renderRequest, renderResponse);
	}
	
	protected String getViewName(RenderRequest renderRequest)
		throws PortletException {
		
		String requestedView = renderRequest.getParameter(REQUESTED_VIEW);
		String goTo = _viewMappings.get(requestedView);
		
		return StringUtils.isBlank(goTo) ? getDefaultView() : goTo;
	}
	
	protected String getDefaultView() throws PortletException {
		String defaultView = _viewMappings.get(DEFAULT_VIEW);
		
		if (StringUtils.isBlank(defaultView)) {
			throw new PortletException("no valid mapping defined for portlet");
		} else {
			return defaultView;
		}
	}
	
	protected void callRenderMethod(
			RenderRequest request, RenderResponse response)
		throws PortletException {

		String requestedView = request.getParameter(REQUESTED_VIEW);
		String viewName =
			Validator.isNull(requestedView) ? DEFAULT_VIEW : requestedView;
		
		try {
			Method method = MethodCache.get(
				_classesMap, _methodsMap, getClass().getName(), viewName,
				new Class[] {RenderRequest.class, RenderResponse.class});

			method.invoke(this, request, response);
		}
		catch (InvocationTargetException ite) {
			Throwable cause = ite.getCause();

			if (cause != null) {
				throw new PortletException(cause);
			}
			else {
				throw new PortletException(ite);
			}
		}
		catch (NoSuchMethodException e) {
			if (_log.isWarnEnabled()) {
				_log.warn(String.format(
						"no render method found for view '%s'. Ignoring.",
						viewName), e);
			}
		}
		catch (Exception e) {
			throw new PortletException(e);
		}
	}
	
	private Map<String,String> _viewMappings;
	private Map<String, Class<?>> _classesMap = new HashMap<String, Class<?>>();
	private Map<MethodKey, Method> _methodsMap =
		new HashMap<MethodKey, Method>();
	
	private static final Log _log = LogFactoryUtil.getLog(
			MultiViewJSPPortlet.class);
}
