/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.ceb2b2000.connector.service.base;

import com.ceb2b2000.connector.model.ContentConnection;
import com.ceb2b2000.connector.service.ContentConnectionLocalService;
import com.ceb2b2000.connector.service.StructureRestrictionLocalService;
import com.ceb2b2000.connector.service.persistence.ContentConnectionPersistence;
import com.ceb2b2000.connector.service.persistence.StructureRestrictionPersistence;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.util.PortalUtil;

import java.util.List;

/**
 * <a href="ContentConnectionLocalServiceBaseImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 *
 */
public abstract class ContentConnectionLocalServiceBaseImpl
	implements ContentConnectionLocalService {
	public ContentConnection addContentConnection(
		ContentConnection contentConnection) throws SystemException {
		contentConnection.setNew(true);

		return contentConnectionPersistence.update(contentConnection, false);
	}

	public ContentConnection createContentConnection(long connectionId) {
		return contentConnectionPersistence.create(connectionId);
	}

	public void deleteContentConnection(long connectionId)
		throws PortalException, SystemException {
		contentConnectionPersistence.remove(connectionId);
	}

	public void deleteContentConnection(ContentConnection contentConnection)
		throws SystemException {
		contentConnectionPersistence.remove(contentConnection);
	}

	public List<Object> dynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		return contentConnectionPersistence.findWithDynamicQuery(dynamicQuery);
	}

	public List<Object> dynamicQuery(DynamicQuery dynamicQuery, int start,
		int end) throws SystemException {
		return contentConnectionPersistence.findWithDynamicQuery(dynamicQuery,
			start, end);
	}

	public ContentConnection getContentConnection(long connectionId)
		throws PortalException, SystemException {
		return contentConnectionPersistence.findByPrimaryKey(connectionId);
	}

	public List<ContentConnection> getContentConnections(int start, int end)
		throws SystemException {
		return contentConnectionPersistence.findAll(start, end);
	}

	public int getContentConnectionsCount() throws SystemException {
		return contentConnectionPersistence.countAll();
	}

	public ContentConnection updateContentConnection(
		ContentConnection contentConnection) throws SystemException {
		contentConnection.setNew(false);

		return contentConnectionPersistence.update(contentConnection, true);
	}

	public ContentConnection updateContentConnection(
		ContentConnection contentConnection, boolean merge)
		throws SystemException {
		contentConnection.setNew(false);

		return contentConnectionPersistence.update(contentConnection, merge);
	}

	public ContentConnectionLocalService getContentConnectionLocalService() {
		return contentConnectionLocalService;
	}

	public void setContentConnectionLocalService(
		ContentConnectionLocalService contentConnectionLocalService) {
		this.contentConnectionLocalService = contentConnectionLocalService;
	}

	public ContentConnectionPersistence getContentConnectionPersistence() {
		return contentConnectionPersistence;
	}

	public void setContentConnectionPersistence(
		ContentConnectionPersistence contentConnectionPersistence) {
		this.contentConnectionPersistence = contentConnectionPersistence;
	}

	public StructureRestrictionLocalService getStructureRestrictionLocalService() {
		return structureRestrictionLocalService;
	}

	public void setStructureRestrictionLocalService(
		StructureRestrictionLocalService structureRestrictionLocalService) {
		this.structureRestrictionLocalService = structureRestrictionLocalService;
	}

	public StructureRestrictionPersistence getStructureRestrictionPersistence() {
		return structureRestrictionPersistence;
	}

	public void setStructureRestrictionPersistence(
		StructureRestrictionPersistence structureRestrictionPersistence) {
		this.structureRestrictionPersistence = structureRestrictionPersistence;
	}

	protected void runSQL(String sql) throws SystemException {
		try {
			PortalUtil.runSQL(sql);
		}
		catch (Exception e) {
			throw new SystemException(e);
		}
	}

	@BeanReference(name = "com.ceb2b2000.connector.service.ContentConnectionLocalService.impl")
	protected ContentConnectionLocalService contentConnectionLocalService;
	@BeanReference(name = "com.ceb2b2000.connector.service.persistence.ContentConnectionPersistence.impl")
	protected ContentConnectionPersistence contentConnectionPersistence;
	@BeanReference(name = "com.ceb2b2000.connector.service.StructureRestrictionLocalService.impl")
	protected StructureRestrictionLocalService structureRestrictionLocalService;
	@BeanReference(name = "com.ceb2b2000.connector.service.persistence.StructureRestrictionPersistence.impl")
	protected StructureRestrictionPersistence structureRestrictionPersistence;
}