/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.ceb2b2000.connector.service.base;

import com.ceb2b2000.connector.model.StructureRestriction;
import com.ceb2b2000.connector.service.ContentConnectionLocalService;
import com.ceb2b2000.connector.service.StructureRestrictionLocalService;
import com.ceb2b2000.connector.service.persistence.ContentConnectionPersistence;
import com.ceb2b2000.connector.service.persistence.StructureRestrictionPersistence;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.util.PortalUtil;

import java.util.List;

/**
 * <a href="StructureRestrictionLocalServiceBaseImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 *
 */
public abstract class StructureRestrictionLocalServiceBaseImpl
	implements StructureRestrictionLocalService {
	public StructureRestriction addStructureRestriction(
		StructureRestriction structureRestriction) throws SystemException {
		structureRestriction.setNew(true);

		return structureRestrictionPersistence.update(structureRestriction,
			false);
	}

	public StructureRestriction createStructureRestriction(long restrictionId) {
		return structureRestrictionPersistence.create(restrictionId);
	}

	public void deleteStructureRestriction(long restrictionId)
		throws PortalException, SystemException {
		structureRestrictionPersistence.remove(restrictionId);
	}

	public void deleteStructureRestriction(
		StructureRestriction structureRestriction) throws SystemException {
		structureRestrictionPersistence.remove(structureRestriction);
	}

	public List<Object> dynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		return structureRestrictionPersistence.findWithDynamicQuery(dynamicQuery);
	}

	public List<Object> dynamicQuery(DynamicQuery dynamicQuery, int start,
		int end) throws SystemException {
		return structureRestrictionPersistence.findWithDynamicQuery(dynamicQuery,
			start, end);
	}

	public StructureRestriction getStructureRestriction(long restrictionId)
		throws PortalException, SystemException {
		return structureRestrictionPersistence.findByPrimaryKey(restrictionId);
	}

	public List<StructureRestriction> getStructureRestrictions(int start,
		int end) throws SystemException {
		return structureRestrictionPersistence.findAll(start, end);
	}

	public int getStructureRestrictionsCount() throws SystemException {
		return structureRestrictionPersistence.countAll();
	}

	public StructureRestriction updateStructureRestriction(
		StructureRestriction structureRestriction) throws SystemException {
		structureRestriction.setNew(false);

		return structureRestrictionPersistence.update(structureRestriction, true);
	}

	public StructureRestriction updateStructureRestriction(
		StructureRestriction structureRestriction, boolean merge)
		throws SystemException {
		structureRestriction.setNew(false);

		return structureRestrictionPersistence.update(structureRestriction,
			merge);
	}

	public ContentConnectionLocalService getContentConnectionLocalService() {
		return contentConnectionLocalService;
	}

	public void setContentConnectionLocalService(
		ContentConnectionLocalService contentConnectionLocalService) {
		this.contentConnectionLocalService = contentConnectionLocalService;
	}

	public ContentConnectionPersistence getContentConnectionPersistence() {
		return contentConnectionPersistence;
	}

	public void setContentConnectionPersistence(
		ContentConnectionPersistence contentConnectionPersistence) {
		this.contentConnectionPersistence = contentConnectionPersistence;
	}

	public StructureRestrictionLocalService getStructureRestrictionLocalService() {
		return structureRestrictionLocalService;
	}

	public void setStructureRestrictionLocalService(
		StructureRestrictionLocalService structureRestrictionLocalService) {
		this.structureRestrictionLocalService = structureRestrictionLocalService;
	}

	public StructureRestrictionPersistence getStructureRestrictionPersistence() {
		return structureRestrictionPersistence;
	}

	public void setStructureRestrictionPersistence(
		StructureRestrictionPersistence structureRestrictionPersistence) {
		this.structureRestrictionPersistence = structureRestrictionPersistence;
	}

	protected void runSQL(String sql) throws SystemException {
		try {
			PortalUtil.runSQL(sql);
		}
		catch (Exception e) {
			throw new SystemException(e);
		}
	}

	@BeanReference(name = "com.ceb2b2000.connector.service.ContentConnectionLocalService.impl")
	protected ContentConnectionLocalService contentConnectionLocalService;
	@BeanReference(name = "com.ceb2b2000.connector.service.persistence.ContentConnectionPersistence.impl")
	protected ContentConnectionPersistence contentConnectionPersistence;
	@BeanReference(name = "com.ceb2b2000.connector.service.StructureRestrictionLocalService.impl")
	protected StructureRestrictionLocalService structureRestrictionLocalService;
	@BeanReference(name = "com.ceb2b2000.connector.service.persistence.StructureRestrictionPersistence.impl")
	protected StructureRestrictionPersistence structureRestrictionPersistence;
}