/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.ceb2b2000.connector.service.impl;

import java.util.Date;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;

import com.ceb2b2000.connector.NoSuchStructureRestrictionException;
import com.ceb2b2000.connector.model.StructureRestriction;
import com.ceb2b2000.connector.service.base.StructureRestrictionLocalServiceBaseImpl;
import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.portal.SystemException;

/**
 * <a href="StructureRestrictionLocalServiceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 *
 */
public class StructureRestrictionLocalServiceImpl
	extends StructureRestrictionLocalServiceBaseImpl {
	
	public List<StructureRestriction> getRestrictions(long companyId, long groupId, String structureId) throws SystemException {
		return getStructureRestrictionPersistence().findByC_G_S(companyId, groupId, structureId);
	}
	
	public void updateRestrictions(
			long companyId, long groupId, long userId,
			String structureId, String[] restrictions) 
		throws SystemException, NoSuchStructureRestrictionException {
		
		List<StructureRestriction> current = getRestrictions(companyId, groupId, structureId);
		
		Collection<String> found = new ArrayList<String>();
		
		for (StructureRestriction restriction: current) {
			if (!ArrayUtils.contains(restrictions, restriction.getRelatedStructureId())) {
				getStructureRestrictionPersistence().remove(restriction.getRestrictionId());
			} else {
				found.add(restriction.getRelatedStructureId());
			}
		}
		
		Collection<String> toAdd = new ArrayList<String>();
		toAdd.addAll(Arrays.asList(restrictions));
		toAdd.removeAll(found);
		
		for (String relatedStructureId: toAdd) {
			Date createdOn = Calendar.getInstance().getTime();
			
			StructureRestriction newRest = createStructureRestriction(CounterLocalServiceUtil.increment(StructureRestriction.class.getName()));
			
			newRest.setCompanyId(companyId);
			newRest.setCreateDate(createdOn);
			newRest.setGroupId(groupId);
			newRest.setModifiedDate(createdOn);
			newRest.setRelatedStructureId(relatedStructureId);
			newRest.setStructureId(structureId);
			newRest.setUserId(userId);
			
			addStructureRestriction(newRest);
		}
	}
}