/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.ceb2b2000.connector.service.persistence;

import com.ceb2b2000.connector.NoSuchStructureRestrictionException;
import com.ceb2b2000.connector.model.StructureRestriction;
import com.ceb2b2000.connector.model.impl.StructureRestrictionImpl;
import com.ceb2b2000.connector.model.impl.StructureRestrictionModelImpl;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.cache.CacheRegistry;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.EntityCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderPath;
import com.liferay.portal.kernel.dao.orm.Query;
import com.liferay.portal.kernel.dao.orm.QueryPos;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.dao.orm.Session;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.model.ModelListener;
import com.liferay.portal.service.persistence.BatchSessionUtil;
import com.liferay.portal.service.persistence.impl.BasePersistenceImpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <a href="StructureRestrictionPersistenceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 *
 */
public class StructureRestrictionPersistenceImpl extends BasePersistenceImpl
	implements StructureRestrictionPersistence {
	public static final String FINDER_CLASS_NAME_ENTITY = StructureRestrictionImpl.class.getName();
	public static final String FINDER_CLASS_NAME_LIST = FINDER_CLASS_NAME_ENTITY +
		".List";
	public static final FinderPath FINDER_PATH_FIND_BY_C_G_S = new FinderPath(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findByC_G_S",
			new String[] {
				Long.class.getName(), Long.class.getName(),
				String.class.getName()
			});
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_C_G_S = new FinderPath(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findByC_G_S",
			new String[] {
				Long.class.getName(), Long.class.getName(),
				String.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_C_G_S = new FinderPath(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "countByC_G_S",
			new String[] {
				Long.class.getName(), Long.class.getName(),
				String.class.getName()
			});
	public static final FinderPath FINDER_PATH_FIND_ALL = new FinderPath(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findAll", new String[0]);
	public static final FinderPath FINDER_PATH_COUNT_ALL = new FinderPath(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "countAll", new String[0]);

	public void cacheResult(StructureRestriction structureRestriction) {
		EntityCacheUtil.putResult(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionImpl.class,
			structureRestriction.getPrimaryKey(), structureRestriction);
	}

	public void cacheResult(List<StructureRestriction> structureRestrictions) {
		for (StructureRestriction structureRestriction : structureRestrictions) {
			if (EntityCacheUtil.getResult(
						StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
						StructureRestrictionImpl.class,
						structureRestriction.getPrimaryKey(), this) == null) {
				cacheResult(structureRestriction);
			}
		}
	}

	public void clearCache() {
		CacheRegistry.clear(StructureRestrictionImpl.class.getName());
		EntityCacheUtil.clearCache(StructureRestrictionImpl.class.getName());
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_ENTITY);
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);
	}

	public StructureRestriction create(long restrictionId) {
		StructureRestriction structureRestriction = new StructureRestrictionImpl();

		structureRestriction.setNew(true);
		structureRestriction.setPrimaryKey(restrictionId);

		return structureRestriction;
	}

	public StructureRestriction remove(long restrictionId)
		throws NoSuchStructureRestrictionException, SystemException {
		Session session = null;

		try {
			session = openSession();

			StructureRestriction structureRestriction = (StructureRestriction)session.get(StructureRestrictionImpl.class,
					new Long(restrictionId));

			if (structureRestriction == null) {
				if (_log.isWarnEnabled()) {
					_log.warn(
						"No StructureRestriction exists with the primary key " +
						restrictionId);
				}

				throw new NoSuchStructureRestrictionException(
					"No StructureRestriction exists with the primary key " +
					restrictionId);
			}

			return remove(structureRestriction);
		}
		catch (NoSuchStructureRestrictionException nsee) {
			throw nsee;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public StructureRestriction remove(
		StructureRestriction structureRestriction) throws SystemException {
		for (ModelListener<StructureRestriction> listener : listeners) {
			listener.onBeforeRemove(structureRestriction);
		}

		structureRestriction = removeImpl(structureRestriction);

		for (ModelListener<StructureRestriction> listener : listeners) {
			listener.onAfterRemove(structureRestriction);
		}

		return structureRestriction;
	}

	protected StructureRestriction removeImpl(
		StructureRestriction structureRestriction) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			if (structureRestriction.isCachedModel() ||
					BatchSessionUtil.isEnabled()) {
				Object staleObject = session.get(StructureRestrictionImpl.class,
						structureRestriction.getPrimaryKeyObj());

				if (staleObject != null) {
					session.evict(staleObject);
				}
			}

			session.delete(structureRestriction);

			session.flush();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.removeResult(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionImpl.class, structureRestriction.getPrimaryKey());

		return structureRestriction;
	}

	public StructureRestriction update(
		StructureRestriction structureRestriction) throws SystemException {
		if (_log.isWarnEnabled()) {
			_log.warn(
				"Using the deprecated update(StructureRestriction structureRestriction) method. Use update(StructureRestriction structureRestriction, boolean merge) instead.");
		}

		return update(structureRestriction, false);
	}

	public StructureRestriction update(
		StructureRestriction structureRestriction, boolean merge)
		throws SystemException {
		boolean isNew = structureRestriction.isNew();

		for (ModelListener<StructureRestriction> listener : listeners) {
			if (isNew) {
				listener.onBeforeCreate(structureRestriction);
			}
			else {
				listener.onBeforeUpdate(structureRestriction);
			}
		}

		structureRestriction = updateImpl(structureRestriction, merge);

		for (ModelListener<StructureRestriction> listener : listeners) {
			if (isNew) {
				listener.onAfterCreate(structureRestriction);
			}
			else {
				listener.onAfterUpdate(structureRestriction);
			}
		}

		return structureRestriction;
	}

	public StructureRestriction updateImpl(
		com.ceb2b2000.connector.model.StructureRestriction structureRestriction,
		boolean merge) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			BatchSessionUtil.update(session, structureRestriction, merge);

			structureRestriction.setNew(false);
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.putResult(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
			StructureRestrictionImpl.class,
			structureRestriction.getPrimaryKey(), structureRestriction);

		return structureRestriction;
	}

	public StructureRestriction findByPrimaryKey(long restrictionId)
		throws NoSuchStructureRestrictionException, SystemException {
		StructureRestriction structureRestriction = fetchByPrimaryKey(restrictionId);

		if (structureRestriction == null) {
			if (_log.isWarnEnabled()) {
				_log.warn(
					"No StructureRestriction exists with the primary key " +
					restrictionId);
			}

			throw new NoSuchStructureRestrictionException(
				"No StructureRestriction exists with the primary key " +
				restrictionId);
		}

		return structureRestriction;
	}

	public StructureRestriction fetchByPrimaryKey(long restrictionId)
		throws SystemException {
		StructureRestriction structureRestriction = (StructureRestriction)EntityCacheUtil.getResult(StructureRestrictionModelImpl.ENTITY_CACHE_ENABLED,
				StructureRestrictionImpl.class, restrictionId, this);

		if (structureRestriction == null) {
			Session session = null;

			try {
				session = openSession();

				structureRestriction = (StructureRestriction)session.get(StructureRestrictionImpl.class,
						new Long(restrictionId));
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (structureRestriction != null) {
					cacheResult(structureRestriction);
				}

				closeSession(session);
			}
		}

		return structureRestriction;
	}

	public List<StructureRestriction> findByC_G_S(long companyId, long groupId,
		String structureId) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId), new Long(groupId),
				
				structureId
			};

		List<StructureRestriction> list = (List<StructureRestriction>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_C_G_S,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"FROM com.ceb2b2000.connector.model.StructureRestriction WHERE ");

				query.append("companyId = ?");

				query.append(" AND ");

				query.append("groupId = ?");

				query.append(" AND ");

				if (structureId == null) {
					query.append("structureId IS NULL");
				}
				else {
					query.append("structureId = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				qPos.add(groupId);

				if (structureId != null) {
					qPos.add(structureId);
				}

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<StructureRestriction>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_C_G_S,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<StructureRestriction> findByC_G_S(long companyId, long groupId,
		String structureId, int start, int end) throws SystemException {
		return findByC_G_S(companyId, groupId, structureId, start, end, null);
	}

	public List<StructureRestriction> findByC_G_S(long companyId, long groupId,
		String structureId, int start, int end, OrderByComparator obc)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId), new Long(groupId),
				
				structureId,
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<StructureRestriction> list = (List<StructureRestriction>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_C_G_S,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"FROM com.ceb2b2000.connector.model.StructureRestriction WHERE ");

				query.append("companyId = ?");

				query.append(" AND ");

				query.append("groupId = ?");

				query.append(" AND ");

				if (structureId == null) {
					query.append("structureId IS NULL");
				}
				else {
					query.append("structureId = ?");
				}

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");
					query.append(obc.getOrderBy());
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				qPos.add(groupId);

				if (structureId != null) {
					qPos.add(structureId);
				}

				list = (List<StructureRestriction>)QueryUtil.list(q,
						getDialect(), start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<StructureRestriction>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_C_G_S,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public StructureRestriction findByC_G_S_First(long companyId, long groupId,
		String structureId, OrderByComparator obc)
		throws NoSuchStructureRestrictionException, SystemException {
		List<StructureRestriction> list = findByC_G_S(companyId, groupId,
				structureId, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No StructureRestriction exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(", ");
			msg.append("groupId=" + groupId);

			msg.append(", ");
			msg.append("structureId=" + structureId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchStructureRestrictionException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public StructureRestriction findByC_G_S_Last(long companyId, long groupId,
		String structureId, OrderByComparator obc)
		throws NoSuchStructureRestrictionException, SystemException {
		int count = countByC_G_S(companyId, groupId, structureId);

		List<StructureRestriction> list = findByC_G_S(companyId, groupId,
				structureId, count - 1, count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No StructureRestriction exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(", ");
			msg.append("groupId=" + groupId);

			msg.append(", ");
			msg.append("structureId=" + structureId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchStructureRestrictionException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public StructureRestriction[] findByC_G_S_PrevAndNext(long restrictionId,
		long companyId, long groupId, String structureId, OrderByComparator obc)
		throws NoSuchStructureRestrictionException, SystemException {
		StructureRestriction structureRestriction = findByPrimaryKey(restrictionId);

		int count = countByC_G_S(companyId, groupId, structureId);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"FROM com.ceb2b2000.connector.model.StructureRestriction WHERE ");

			query.append("companyId = ?");

			query.append(" AND ");

			query.append("groupId = ?");

			query.append(" AND ");

			if (structureId == null) {
				query.append("structureId IS NULL");
			}
			else {
				query.append("structureId = ?");
			}

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");
				query.append(obc.getOrderBy());
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(companyId);

			qPos.add(groupId);

			if (structureId != null) {
				qPos.add(structureId);
			}

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					structureRestriction);

			StructureRestriction[] array = new StructureRestrictionImpl[3];

			array[0] = (StructureRestriction)objArray[0];
			array[1] = (StructureRestriction)objArray[1];
			array[2] = (StructureRestriction)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery,
		int start, int end) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.setLimit(start, end);

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<StructureRestriction> findAll() throws SystemException {
		return findAll(QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}

	public List<StructureRestriction> findAll(int start, int end)
		throws SystemException {
		return findAll(start, end, null);
	}

	public List<StructureRestriction> findAll(int start, int end,
		OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<StructureRestriction> list = (List<StructureRestriction>)FinderCacheUtil.getResult(FINDER_PATH_FIND_ALL,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"FROM com.ceb2b2000.connector.model.StructureRestriction ");

				if (obc != null) {
					query.append("ORDER BY ");
					query.append(obc.getOrderBy());
				}

				Query q = session.createQuery(query.toString());

				if (obc == null) {
					list = (List<StructureRestriction>)QueryUtil.list(q,
							getDialect(), start, end, false);

					Collections.sort(list);
				}
				else {
					list = (List<StructureRestriction>)QueryUtil.list(q,
							getDialect(), start, end);
				}
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<StructureRestriction>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_ALL, finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public void removeByC_G_S(long companyId, long groupId, String structureId)
		throws SystemException {
		for (StructureRestriction structureRestriction : findByC_G_S(
				companyId, groupId, structureId)) {
			remove(structureRestriction);
		}
	}

	public void removeAll() throws SystemException {
		for (StructureRestriction structureRestriction : findAll()) {
			remove(structureRestriction);
		}
	}

	public int countByC_G_S(long companyId, long groupId, String structureId)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId), new Long(groupId),
				
				structureId
			};

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_C_G_S,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(*) ");
				query.append(
					"FROM com.ceb2b2000.connector.model.StructureRestriction WHERE ");

				query.append("companyId = ?");

				query.append(" AND ");

				query.append("groupId = ?");

				query.append(" AND ");

				if (structureId == null) {
					query.append("structureId IS NULL");
				}
				else {
					query.append("structureId = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				qPos.add(groupId);

				if (structureId != null) {
					qPos.add(structureId);
				}

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_C_G_S,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countAll() throws SystemException {
		Object[] finderArgs = new Object[0];

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_ALL,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				Query q = session.createQuery(
						"SELECT COUNT(*) FROM com.ceb2b2000.connector.model.StructureRestriction");

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_ALL, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public void afterPropertiesSet() {
		String[] listenerClassNames = StringUtil.split(GetterUtil.getString(
					com.liferay.util.service.ServiceProps.get(
						"value.object.listener.com.ceb2b2000.connector.model.StructureRestriction")));

		if (listenerClassNames.length > 0) {
			try {
				List<ModelListener<StructureRestriction>> listenersList = new ArrayList<ModelListener<StructureRestriction>>();

				for (String listenerClassName : listenerClassNames) {
					listenersList.add((ModelListener<StructureRestriction>)Class.forName(
							listenerClassName).newInstance());
				}

				listeners = listenersList.toArray(new ModelListener[listenersList.size()]);
			}
			catch (Exception e) {
				_log.error(e);
			}
		}
	}

	@BeanReference(name = "com.ceb2b2000.connector.service.persistence.ContentConnectionPersistence.impl")
	protected com.ceb2b2000.connector.service.persistence.ContentConnectionPersistence contentConnectionPersistence;
	@BeanReference(name = "com.ceb2b2000.connector.service.persistence.StructureRestrictionPersistence.impl")
	protected com.ceb2b2000.connector.service.persistence.StructureRestrictionPersistence structureRestrictionPersistence;
	private static Log _log = LogFactoryUtil.getLog(StructureRestrictionPersistenceImpl.class);
}