package com.ceb2b2000.connector.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import com.ceb2b2000.connector.connectors.Connectable;
import com.ceb2b2000.connector.connectors.ConnectableFactory;
import com.ceb2b2000.connector.connectors.ConnectableUtil;
import com.ceb2b2000.connector.connectors.ConnectorContext;
import com.ceb2b2000.connector.connectors.ConnectorException;
import com.ceb2b2000.connector.model.ContentConnection;
import com.ceb2b2000.connector.service.ContentConnectionLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Field;

public final class ConnectableCollectionUtil {
	private ConnectableCollectionUtil() {}
	
	public static final <T> List<Connectable> asConnectables(
			ConnectorContext ctx, Collection<T> models,
			ConnectableFactory<T> factory) {
		
		List<Connectable> entities = 
			new ArrayList<Connectable>(models.size());
		
		for (T model: models) {
			entities.add(factory.fromModel(ctx, model, 0L));
		}
		
		return entities;
	}
	
	public static final <T> List<Connectable> asConnectables(
			ConnectorContext ctx, Document[] documents,
			ConnectableFactory<T> factory, Long... excluded)
		throws PortalException, SystemException {
		
		List<Connectable> entities = 
			new ArrayList<Connectable>(documents.length);
		
		if (excluded == null || excluded.length == 0) {
			convertAllToEntities(ctx, documents, factory, entities);
		} else if (excluded.length == 1) {
			convertAllExceptOne(ctx, documents, factory, entities, excluded[0]);
		} else {
			convertAllExcept(
					ctx, documents, factory, entities, Arrays.asList(excluded));
		}
		
		return entities;
	}

	private static final <T> void convertAllExcept(
			ConnectorContext ctx, Document[] documents,
			ConnectableFactory<T> factory,
			List<Connectable> entities, Collection<Long> excluded)
		throws PortalException, SystemException {
		
		for (Document doc: documents) {
			if (!excluded.contains(Long.parseLong(doc.get(Field.ENTRY_CLASS_PK)))) {
				entities.add(factory.fromHit(ctx, doc, 0L));
			}
		}
	}

	private static final <T> void convertAllExceptOne(
			ConnectorContext ctx, Document[] documents,
			ConnectableFactory<T> factory,
			Collection<Connectable> entities, long excluded)
		throws PortalException, SystemException {
		
		for (Document doc: documents) {
			if (excluded <= 0 || 
					excluded != Long.parseLong(doc.get(Field.ENTRY_CLASS_PK))) {
				entities.add(factory.fromHit(ctx, doc, 0L));
			}
		}
	}

	private static <T> void convertAllToEntities(
			ConnectorContext ctx, Document[] documents,
			ConnectableFactory<T> factory,
			Collection<Connectable> entities)
		throws PortalException,	SystemException {

		for (Document doc: documents) {
			entities.add(factory.fromHit(ctx, doc, 0L));
		}
	}
	
	public static final <T> List<Connectable> fromConnectionHits(
			ConnectorContext ctx, Document[] connectionDocuments) 
		throws PortalException, SystemException, ConnectorException {
		
		List<Connectable> entities = new ArrayList<Connectable>(
				connectionDocuments.length);
		
		Set<Long> found = new TreeSet<Long>();
		
		for (Document doc: connectionDocuments) {
			long connectionId = Long.parseLong(doc.get(Field.ENTRY_CLASS_PK));
			
			ContentConnection connection =
				ContentConnectionLocalServiceUtil.getContentConnection(
						connectionId);
			
			if (!found.contains(connection.getPrimaryId())) {
				entities.add(ConnectableUtil.getPrimaryEntity(ctx, connection));
				found.add(connection.getPrimaryId());
			}
		}
		
		return entities;
	}
	
	public static final <T> List<Connectable> asSecondaryEntityList(
			ConnectorContext ctx, Collection<ContentConnection> connections,
			ConnectableFactory<T> factory)
		throws PortalException, SystemException, ConnectorException {
		
		List<Connectable> entities = new ArrayList<Connectable>(
				connections.size());

		for (ContentConnection connection : connections) {
			entities.add(ConnectableUtil.getSecondaryEntity(ctx, connection));
		}
		
		return entities;
	}
	
	public static final <T> List<Connectable> asSecondaryEntityList(
			ConnectorContext ctx, Document[] documents,
			ConnectableFactory<T> factory)
		throws PortalException, SystemException, ConnectorException {
		
		List<Connectable> entities = new ArrayList<Connectable>(
				documents.length);

		for (Document doc : documents) {
			long connectionId = Long.parseLong(doc.get(Field.ENTRY_CLASS_PK));
			
			ContentConnection connection =
				ContentConnectionLocalServiceUtil.getContentConnection(
						connectionId);
			
			entities.add(ConnectableUtil.getSecondaryEntity(ctx, connection));
		}
		
		return entities;
	}
}
