package com.ceb2b2000.connector.util;

import javax.portlet.PortletURL;

import com.ceb2b2000.connector.model.ContentConnection;
import com.ceb2b2000.connector.service.ContentConnectionLocalServiceUtil;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.DocumentImpl;
import com.liferay.portal.kernel.search.DocumentSummary;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.search.SearchEngineUtil;
import com.liferay.portal.kernel.search.SearchException;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portlet.expando.model.ExpandoBridge;
import com.liferay.portlet.expando.util.ExpandoBridgeIndexerUtil;

public class Indexer implements com.liferay.portal.kernel.search.Indexer {
	public static final String PORTLET_ID = "content-connector";
	
	public static final void addConnection(
			long companyId, long groupId, long userId, long connectionId,
			long primaryId, long primaryClassId, String primaryKeywords, 
			long secondaryId, long secondaryClassId, String secondaryKeywords,
			String[] tagsEntries, ExpandoBridge expandoBridge) 
		throws SearchException {
		
		Document doc = getConnectionDocument(
				companyId, groupId, userId, connectionId, 
				primaryId, primaryClassId, primaryKeywords, 
				secondaryId, secondaryClassId, secondaryKeywords, 
				tagsEntries, expandoBridge);
		
		SearchEngineUtil.addDocument(companyId, doc);
	}
	
	public static final void deleteConnection(long companyId, long connectionId)
		throws SearchException {
		
		SearchEngineUtil.deleteDocument(
				companyId, getConnectionUID(connectionId));
	}
	
	public static final void updateConnection(
			long companyId, long groupId, long userId, long connectionId,
			long primaryId, long primaryClassId, String primaryKeywords,
			long secondaryId, long secondaryClassId, String secondaryKeywords,
			String[] tagsEntries, ExpandoBridge expandoBridge) 
		throws SearchException {
		
		Document doc = getConnectionDocument(
				companyId, groupId, userId, connectionId,
				primaryId, primaryClassId, primaryKeywords, 
				secondaryId, secondaryClassId, secondaryKeywords,
				tagsEntries, expandoBridge);
		
		SearchEngineUtil.updateDocument(companyId, doc.get(Field.UID), doc);
	}
	
	private static final String getConnectionUID(long connectionId) {
		Document doc = new DocumentImpl();
		
		doc.addUID(PORTLET_ID, connectionId);
		
		return doc.get(Field.UID);
	}
	
	public static final Document getConnectionDocument(
			long companyId, long groupId, long userId, long connectionId,
			long primaryId, long primaryClassId, String primaryKeywords,
			long secondaryId, long secondaryClassId, String secondaryKeywords,
			String[] tagsEntries, ExpandoBridge expandoBridge	) {
		
		Document doc = new DocumentImpl();
		
		doc.addUID(PORTLET_ID, connectionId);
		
		doc.addModifiedDate();
		
		doc.addKeyword(Field.COMPANY_ID, companyId);
		doc.addKeyword(Field.GROUP_ID, groupId);
		doc.addKeyword(Field.USER_ID, userId);
		doc.addKeyword(Field.PORTLET_ID, PORTLET_ID);
		
		doc.addKeyword(Field.ENTRY_CLASS_PK, connectionId);
		doc.addKeyword(Field.ENTRY_CLASS_NAME, ContentConnection.class.getName());
		
		doc.addKeyword(ConnectorField.PRIMARY_CLASS_PK, primaryId);
		doc.addKeyword(ConnectorField.PRIMARY_CLASS_NAME_ID, primaryClassId);
		doc.addText(ConnectorField.PRIMARY_KEYWORDS, primaryKeywords);

		doc.addKeyword(ConnectorField.SECONDARY_CLASS_PK, secondaryId);
		doc.addKeyword(ConnectorField.SECONDARY_CLASS_NAME_ID, secondaryClassId);
		doc.addText(ConnectorField.SECONDARY_KEYWORDS, secondaryKeywords);

		doc.addKeyword(Field.TAGS_ENTRIES, tagsEntries);
		
		ExpandoBridgeIndexerUtil.addAttributes(doc, expandoBridge);
		
		return doc;
		
	}
	
	public String[] getClassNames() {
		return CLASS_NAMES;
	}

	public DocumentSummary getDocumentSummary(
			Document doc, PortletURL portletURL) {
		
		String title = doc.get(Field.TITLE);
		String description = StringUtil.shorten(doc.get(Field.DESCRIPTION));
		
		return new DocumentSummary(title, description, null);
	}

	public void reIndex(String className, long classPK) throws SearchException {
		ContentConnectionLocalServiceUtil.reIndex(className, classPK);
	}

	public void reIndex(String[] ids) throws SearchException {
		ContentConnectionLocalServiceUtil.reIndex(ids);
	}

	private static final String[] CLASS_NAMES = { 
		ContentConnection.class.getName() 
	};
}
