package com.ceb2b2000.connector.util;

import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.DynamicQueryFactoryUtil;
import com.liferay.portal.kernel.dao.orm.OrderFactoryUtil;
import com.liferay.portal.kernel.dao.orm.ProjectionFactoryUtil;
import com.liferay.portal.kernel.dao.orm.PropertyFactoryUtil;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portlet.journal.model.JournalStructure;
import com.liferay.portlet.journal.service.JournalStructureLocalServiceUtil;
import com.liferay.support.tomcat.loader.PortalClassLoaderFactory;

public final class JournalStructureUtil {
	private JournalStructureUtil() {}
	
	public static final List<JournalStructure> getAllStructures(
			long companyId, long groupId)
		throws SystemException {
		
		return JournalStructureLocalServiceUtil.search(
				companyId, groupId, "%", "%", "%", false, 
				QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}
	
	public static final List<JournalStructure> getCurrentStructures(
			long companyId, long groupId, String[] currentIds)
		throws SystemException {
		
		if (ArrayUtils.isEmpty(currentIds)) {
			return Collections.emptyList();
		} else {
			List result = JournalStructureLocalServiceUtil.dynamicQuery(
					getCurrentQuery(companyId, groupId, currentIds));
			
			return result;
		}
	}
	
	public static final List<JournalStructure> getAvailableStructures(
			long companyId, long groupId, String[] currentIds) 
		throws SystemException {
		
		if (ArrayUtils.isEmpty(currentIds)) {
			return getAllStructures(companyId, groupId);
		} else {
			List result = JournalStructureLocalServiceUtil.dynamicQuery(
					getAvailableQuery(companyId, groupId, currentIds));
		
			return result;
		}
	}
	
	private static final DynamicQuery getCurrentQuery(
			long companyId, long groupId, String[] currentIds) {
		
		DynamicQuery query = DynamicQueryFactoryUtil.forClass(
				JournalStructure.class, PortalClassLoaderFactory.getClassLoader());
		
		query.add(PropertyFactoryUtil.forName("companyId").eq(companyId));
		query.add(PropertyFactoryUtil.forName("groupId").eq(groupId));
		
		if (currentIds != null && currentIds.length > 0) {
			query.add(PropertyFactoryUtil.forName("structureId").in(currentIds));
		}
		
		query.addOrder(OrderFactoryUtil.asc("structureId"));
		
		return query;
	}
	
	private static final DynamicQuery getAvailableQuery(
			long companyId, long groupId, String[] currentIds) {
		
		DynamicQuery query = DynamicQueryFactoryUtil.forClass(
				JournalStructure.class, PortalClassLoaderFactory.getClassLoader());
		
		query.add(PropertyFactoryUtil.forName("companyId").eq(companyId));
		query.add(PropertyFactoryUtil.forName("groupId").eq(groupId));
		
		if (currentIds != null && currentIds.length > 0) {
			query.add(PropertyFactoryUtil.forName("structureId").notIn(
					getCurrentQuery(companyId, groupId, currentIds)
					.setProjection(ProjectionFactoryUtil.property("structureId"))));
		}

		query.addOrder(OrderFactoryUtil.asc("structureId"));
		
		return query;
	}
}
