package com.ceb2b2000.connector.connectors.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.Properties;

import org.junit.Before;
import org.junit.Test;

import com.ceb2b2000.connector.connectors.ConnectorConfiguration;
import com.ceb2b2000.connector.connectors.ConnectorFactory;
import com.ceb2b2000.connector.connectors.Connector;

public class DefaultConnectorFactoryTest {
	private ConnectorFactory factory;
	
	@Before
	public void setUp() {
		Properties config = new Properties();
		
		config.put("content.connector.supported.types", "java.lang.Long");
		config.put("java.lang.Long.connector.class", CalEventConnector.class.getName());

		
		ConnectorConfiguration connectorProps = new ConnectorConfiguration(config);
		
		factory = new DefaultConnectorFactory(connectorProps);
	}

	@Test
	public void tryNullEntityClass() {
		assertNull(factory.getConnector((Class<?>)null));
	}
	
	@Test
	public void tryNullEntityId() {
		assertNull(factory.getConnector((String)null));
	}
	
	@Test
	public void tryUnsupportedEntityClass() {
		assertNull(factory.getConnector(java.lang.Integer.class));
	}
	
	@Test
	public void supportedEntityClass() {
		Connector<?> c = (Connector<?>) factory.getConnector(java.lang.Long.class);
		
		assertNotNull(c);
	}
	
	@Test
	public void getAll() {
		Collection<Connector<?>> connectors = factory.getAll();
		
		assertEquals(1, connectors.size());
		assertTrue(connectors.iterator().next() instanceof CalEventConnector);
	}
}
